#include <REGX51.H>

/*
 * Filename : lcd.h
 * Hardware : Controller  -> P89V51RD2
 *            XTAL        -> 18.432 MHz
 *            Mode        -> 6 Clock/MC
 * I/O      : RS          -> P2.5
 *            Enable      -> P2.4
 *            Data4567    -> P2.0,P2.1,P2.2,P2.3
 * Compiler : SDCC            
 * Author   : sci-3d@hotmail.com
 * Date		: 01/08/06
 */
/*
Line diagram :
mega32            LCD Module
P2_7->D0
P2_6->D1
P2_5->D2
P2_4->D3
P2_3->D4
P2_2->D5
P2_1->D6
P2_0->D7

P0_2->RS
P0_3->RW
P0_4->En




DDRAM Address :
0x80 to 0x8F  =  1st Line
0xC0 to 0xCF  =  2nd Line

*/

 #define LCD_en P0_4
 #define LCD_rs P0_2 
 #define LCD_rw P0_3 
 #define LCD_DELAY 1000/* Delay for 1 ms */
 #define LCD_clear() LCD_command(0x01)	/* Clear display LCD */
 #define LCD_origin() LCD_command(0x02)	/* Set to origin LCD */
 #define LCD_row1() LCD_command(0x80)	/* Begin at Line 1 */
 #define LCD_row2() LCD_command(0xC0)   /* Begin at Line 2 */
 
/***************************************************
 * Prototype(s)                                    *
 ***************************************************/
void LCD_delay(unsigned char ms);
void LCD_enable();
void LCD_command(unsigned char command);
void LCD_putc(unsigned char ascii);
void LCD_puts(unsigned char *lcd_string);
void LCD_init();
void serial_init(){
        TMOD = 0x20;
        SCON = 0x50;
        TH1  = 0xFD;
        TL1  = 0xFD;
        TR1  = 1;
}
/*void serial_send(unsigned char dat)
{
        while(!TI);
        TI = 0;
        SBUF = dat;
}
*/
unsigned char serial_read(){
        while(!RI);
        RI = 0;
        return SBUF;
}

void main()
{

unsigned int i;
unsigned char a;
LCD_init();
LCD_delay(5);

serial_init();

for(i=0;i<1000;i++)
{
}
LCD_clear();
LCD_delay(5);
LCD_origin();
LCD_delay(5);
LCD_row1();
LCD_delay(5);
while(1)
{
	a=serial_read();
		for(i=0;i<1000;i++)
		{
		}
	LCD_putc(a);
	LCD_delay(5000);
	LCD_putc('H');
	LCD_delay(5000);

}
}
/***************************************************
 * Sources                                         *
 ***************************************************/
void LCD_delay(unsigned char ms)
{
	unsigned char n;
	unsigned int i;
	for (n=0; n<ms; n++)
	{
		for (i=0; i<LCD_DELAY; i++); /* For 1 ms */
	}
    
}

void LCD_enable()
{
    LCD_en = 0; /* Clear bit P2.4 */
    LCD_delay(5);
    LCD_en = 1; /* Set bit P2.4 */
}

void LCD_command(unsigned char command)
{    unsigned char c2;
   c2=command;
    LCD_rs = 0; 
	LCD_delay(5);
   P2_0  =(c2&0x80)>>7;
	P2_1=(c2&0x40)>>6;
    P2_2=(c2&0x20)>>5;
	P2_3=(c2&0x10)>>4;
	P2_4=(c2&0x08)>>3;
	P2_5=(c2&0x04)>>2;
	P2_6=(c2&0x02)>>1;
	P2_7=(c2&0x01)>>0;

	 LCD_rw = 0;
	 LCD_delay(5);
   /* P2 = (P2 & 0xF0)|((command>>4) & 0x0F);
    LCD_enable();
    P2 = (P2 & 0xF0)|(command & 0x0F);*/
    LCD_enable();
    LCD_delay(5);
}

void LCD_putc(unsigned char ascii)
{   unsigned char c1;
c1=ascii;
    LCD_rs = 1; 
    P2_0=(c1&0x80)>>7;
	P2_1=(c1&0x40)>>6;
    P2_2=(c1&0x20)>>5;
	P2_3=(c1&0x10)>>4;
	P2_4=(c1&0x08)>>3;
	P2_5=(c1&0x04)>>2;
	P2_6=(c1&0x02)>>1;
	P2_7=(c1&0x01)>>0;


	LCD_rw = 0;
   /* P2 = (P2 & 0xF0)|((ascii>>4) & 0x0F);
    LCD_enable();
    P2 = (P2 & 0xF0)|(ascii & 0x0F);*/
    LCD_enable();
    LCD_delay(5);
}

void LCD_puts(unsigned char *lcd_string)
{
	while (*lcd_string) 
	{
		LCD_putc(*lcd_string++);
	}
}

void LCD_init()
{
    LCD_delay(5);
    LCD_en = 1;
    /* Set bit P2.4 */
    LCD_delay(5);
	LCD_rs = 0; /* Clear bit P2.5 */ 
    LCD_rw = 0;
    LCD_delay(5);
	LCD_command(0x33);
    //LCD_command(0x32);
    LCD_command(0x38);
    LCD_command(0x0C);
    LCD_command(0x06);
    LCD_command(0x01); /* Clear */
    LCD_delay(5);
}



